/*!
 * Ghostery for Firefox
 * http://www.ghostery.com/
 *
 * Copyright 2013 EVIDON, Inc. All rights reserved.
 * See ghostery_eula.txt for license.
 */

var utils = require('./utils'),
	bugDb = require('./bugdb').load,
	lsoDb = require('./lsodb').load;

function processUrl(src) {
	var q,
		processedUrl,
		src_host,
		src_path;

	// strip out the querystring, including the ?, to reduce false positives
	q = src.indexOf('?');
	if (q >= 0) {
		src = src.slice(0, q);
	}

	// strip out the schema
	// NOTE check for placement of schema since there could be one in the path, in which case this would fail
	// NOTE we check for schema on the first 6 characters because we only care about file, http or https schemas.
	q = src.indexOf('://');
	if (q >= 0 && q < 6) {
		src = src.slice(q + 3);
	}

	// strip out agnostic scheme
	q = src.indexOf('//');
	if (q == 0) {
		src = src.slice(q + 2);
	}

	// lower case
	src = src.toLowerCase();

	q = src.indexOf('/');
	// host should be everything from the start until the first "/"
	src_host = (q >= 0) ? src.substr(0, q) : src;
	// path should be everything after the first "/"
	src_path = (q >= 0) ? src.substr(q + 1) : '';

	// remove port from src_host
	q = src_host.indexOf(':');
	src_host = (q >= 0) ? src_host.substr(0, q) : src_host;

	processedUrl = function () {
		return src;
	};
	processedUrl.host = src_host;
	processedUrl.path = src_path;

	return processedUrl;
}

function matchesHostPath(root, src_path) {
	var path;

	if (!root.hasOwnProperty('$')) { return false; }

	for (path in root.$) {
		if (src_path.indexOf(path) === 0) {
			return root.$[path];
		}
	}

	return false;
}

function matchesHost(root, src_host, src_path) {
	var host_rev_arr = src_host.split('.').reverse(),
		part,
		node = root,
		i;

	for (i = 0; i < host_rev_arr.length; i++) {
		part = host_rev_arr[i];

		// if the node does not have a matching child
		if (!node.hasOwnProperty(part)) {

			// handle path
			if (src_path !== undefined) {
				return matchesHostPath(node, src_path);
			}

			// check if there is a valid match on the node
			return (node.$ !== undefined) ? node.$ : false;
		} else

		// if we are on the last "part"
		if (i == host_rev_arr.length - 1) {

			// handle path
			if (src_path !== undefined) {
				return matchesHostPath(node[part], src_path) || matchesHostPath(node, src_path);
			}

			// check if there is a valid match on the node's child [part]
			if (node[part].$ !== undefined) {
				return node[part].$;

			// else, go back to the parent node and check for a valid match
			// NOTE this is added to handle situations where a host+path is entered
			// with a sub domain and a host is entered without a sub domain
			// (i.e. assets.pinterest.com/images/PinExt.png and pinterest.com)
			} else {
				return (node.$ !== undefined) ? node.$ : false;
			}
		}

		// advance node
		node = node[part];
	}

	return false;
}

function matchesRegEx(src) {
	for (var bug_id in bugDb.db.HASH.regex) {
		if (bugDb.db.HASH.regex[bug_id].test(src)) {
			return +bug_id;
		}
	}

	return false;
}

function matchesPath(src_path) {
	src_path = '/' + src_path;

	for (var path in bugDb.db.HASH.path) {
		if (src_path.indexOf(path) >= 0) {
			return bugDb.db.HASH.path[path];
		}
	}

	return false;
}

// TODO THIS HAS TO BE SUPER FAST
function isBug(src, tab_url) {
	var db = bugDb.db,
		found = false;

	src = processUrl(src);

	found =
		// 2. check host+path hash
		matchesHost(db.HASH.host_path, src.host, src.path) ||
		// 1. check host hash
		matchesHost(db.HASH.host, src.host) ||
		// 3. check path hash
		matchesPath(src.path) ||
		// 4. check regex patterns
		matchesRegEx(src());

	// check firstPartyExceptions
	if (found !== false &&
		db.firstPartyExceptions[found] &&
		utils.fuzzyUrlMatcher(tab_url, db.firstPartyExceptions[found])) {
		return false;
	}

	return found;
}

// TODO THIS HAS TO BE SUPER FAST
function isLso(src) {
	var i,
		id,
		priorities = ['high', 'regular', 'low'],
		regexes,
		db = lsoDb.db,
		q;

	if (!db.fullRegex.test(src)) {
		return false;
	}

	// strip out the querystring, including the ?, to reduce false positives
	q = src.indexOf('?');
	if (q >= 0) {
		src = src.slice(0, q);
	}

	for (i = 0; i < priorities.length; i++) {
		regexes = db.regexes[priorities[i]];

		for (id in regexes) {
			// note: no hasOwnProperty check since our code is sandboxed

			// TODO can still produce false positives (when something that
			// TODO matches a tracker is in the path somewhere, for example)
			if (regexes[id].test(src)) {
				return id;
			}
		}
	}

	return false;
}

exports.isBug = isBug;
exports.isLso = isLso;
